package com.ekexiu.portal.dao;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.orm.annotation.dao.DAO;
import org.jfw.apt.orm.annotation.dao.method.From;
import org.jfw.apt.orm.annotation.dao.method.OrderBy;
import org.jfw.apt.orm.annotation.dao.method.operator.Insert;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectList;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectOne;
import org.jfw.apt.orm.annotation.dao.method.operator.UpdateWith;
import org.jfw.apt.orm.annotation.dao.param.Set;
import org.jfw.util.PageQueryResult;

import com.ekexiu.portal.po.Consult;
import com.ekexiu.portal.pojo.EditProfessor;

@DAO
public abstract class ConsultDao {
	@Autowrie
	private ProfessorDao professorDao;
	
	public ProfessorDao getProfessorDao() {
		return professorDao;
	}

	public void setProfessorDao(ProfessorDao professorDao) {
		this.professorDao = professorDao;
	}

	@Insert
	public abstract int insert(Connection con, Consult consult) throws SQLException;
	
	@UpdateWith
	@From(Consult.class)
	public abstract int updateFinishTime(Connection con,String consultId,@Set String finishTime)throws SQLException;
	
	@UpdateWith
	@From(Consult.class)
	public abstract int updateConsultStatus(Connection con,String consultId,@Set Integer consultStatus)throws SQLException;
	
	public int updateReadStatus(Connection con,String consultId) throws SQLException{
        int index = 1;
        String sql ="UPDATE CONSULT SET READ_STATUS = 1 WHERE CONSULT_ID = ? ";
        PreparedStatement ps = con.prepareStatement(sql);
        try{
            ps.setString(index++,consultId);
            return ps.executeUpdate();
        }finally{
            try{ps.close();}catch(Exception e){}
        }
    }
	
	/**
	 * 更新完成时间和完成状态
	 * @param con 
	 * @param consultId 咨询ID
	 * @param consultStatus 咨询状态
	 * @return 
	 * @throws SQLException
	 */
	public int updateFinishTime(Connection con,String consultId,int consultStatus) throws SQLException{
        int index = 1;
        String sql ="UPDATE CONSULT SET FINISH_TIME= TO_CHAR(NOW(),'YYYYMMDDHH24MISS'),CONSULT_STATUS= ? WHERE CONSULT_ID = ?";
        PreparedStatement ps = con.prepareStatement(sql);
        try{
        	ps.setInt(index++, consultStatus);
            ps.setString(index++,consultId);
            return ps.executeUpdate();
        }finally{
            try{ps.close();}catch(Exception e){}
        }
    }
	
	public int updateRevoveryTime(Connection con,String consultId) throws SQLException{
        int index = 1;
        String sql ="UPDATE CONSULT SET REVOVERY_TIME= TO_CHAR(NOW(),'YYYYMMDDHH24MISS') WHERE CONSULT_ID = ?";
        PreparedStatement ps = con.prepareStatement(sql);
        try{
            ps.setString(index++,consultId);
            return ps.executeUpdate();
        }finally{
            try{ps.close();}catch(Exception e){}
        }
    }
	
	public int updateAssess(Connection con, String consultId, int assessStatus, int assessStar, String assessContant) throws SQLException{
        int index = 1;
        boolean hasAssessContant = null == assessContant;
        String sql ="UPDATE CONSULT SET ASSESS_TIME= TO_CHAR(NOW(),'YYYYMMDDHH24MISS'),ASSESS_CONTANT= ?,ASSESS_STAR= ?,ASSESS_STATUS= ? WHERE CONSULT_ID = ?";
        PreparedStatement ps = con.prepareStatement(sql);
        try{
            if(hasAssessContant){
                ps.setNull(index++,12);
            }else{
                ps.setString(index++,assessContant);
            }
            ps.setInt(index++,assessStar);
            ps.setInt(index++, assessStatus);
            ps.setString(index++,consultId);
            return ps.executeUpdate();
        }finally{
            try{ps.close();}catch(Exception e){}
        }
    }
	
	public int updateThanks(Connection con,String consultId,int thanksStatus,BigDecimal thanksMoney) throws SQLException{
        int index = 1;
        boolean hasThanksMoney = null == thanksMoney;
        String sql ="UPDATE CONSULT SET THANKS_TIME= TO_CHAR(NOW(),'YYYYMMDDHH24MISS'),THANKS_MONEY= ?,THANKS_STATUS= ? WHERE CONSULT_ID = ?";
        PreparedStatement ps = con.prepareStatement(sql);
        try{
            if(hasThanksMoney){
                ps.setNull(index++,3);
            }else{
                ps.setBigDecimal(index++,thanksMoney);
            }
            ps.setInt(index++,thanksStatus);
            ps.setString(index++,consultId);
            return ps.executeUpdate();
        }finally{
            try{ps.close();}catch(Exception e){}
        }
    }
	
	/**
	 * 通过需求ID和专家ID查询咨询表中是否有该专家接受需求的咨询记录
	 * @param con
	 * @param professorId 专家ID
	 * @param demandId 需求ID
	 * @return 若有记录返回咨询ID，否则返回null
	 * @throws SQLException
	 */
	public String queryDemand(Connection con,String professorId,String demandId) throws SQLException{
        int index = 1;
        String sql = "SELECT CONSULT_ID FROM CONSULT WHERE DEMAND_ID = ? AND PROFESSOR_ID = ?";
        PreparedStatement ps = con.prepareStatement(sql);
        try{
            ps.setString(index++,demandId);
            ps.setString(index++,professorId);
            ResultSet rs = ps.executeQuery();
            try{
                if(rs.next()){
                    return rs.getString(1);
                }else{
                    return null;
                }
            }finally{
                try{rs.close();}catch(Exception e1){}
            }
        }finally{
            try{ps.close();}catch(Exception e2){}
        }
    }
	
	/**
	 * 查询当前用户接受咨询被评价的平均星级
	 * @param con
	 * @param professorId 用户ID
	 * @return 返回用户被评价的平均星级
	 * @throws SQLException
	 */
	public BigDecimal queryStarLevel(Connection con, String professorId) throws SQLException {
		int index = 1;
		String sql = " SELECT AVG(ASSESS_STAR) FROM CONSULT WHERE ASSESS_STATUS = 1 AND PROFESSOR_ID = ? ";
		PreparedStatement ps = con.prepareStatement(sql);
		try {
			ps.setString(index++, professorId);
			ResultSet rs = ps.executeQuery();
			try{
				rs.next();
				BigDecimal avg = rs.getBigDecimal(1);
				return avg;
			}finally{
                try{rs.close();}catch(Exception e1){}
            }
		} finally{
            try{ps.close();}catch(Exception e2){}
        }
	}
	
	/**
	 * 查询专家合作历史及评价接口
	 * @param con
	 * @param professorId 专家ID
	 * @param pageSize
	 * @param pageNo
	 * @return 返回所有已评价的咨询及评价详情
	 * @throws SQLException
	 */
	public PageQueryResult<Consult> queryAssessHis(Connection con,String professorId,int pageSize,int pageNo) throws SQLException{
        int total = 0;
        PageQueryResult<Consult> queryResult = new PageQueryResult<Consult>();
        int index = 1;
        String sql = null;
        sql = "  WHERE PROFESSOR_ID = ? AND ASSESS_STATUS = 1 ";
        String whereSql = sql;
        sql = "SELECT COUNT(1) FROM CONSULT";
        sql = sql + whereSql;
        PreparedStatement ps = con.prepareStatement(sql);
        try{
            ps.setString(index++,professorId);
            queryResult.setPageSize(pageSize);
            ResultSet rs = ps.executeQuery();
            try{
                rs.next();
                total = rs.getInt(1);
            }finally{
                try{rs.close();}catch(Exception e1){}
            }
        }finally{
            try{ps.close();}catch(Exception e2){}
        }
        queryResult.setTotal(total);
        if(0== total){
        	queryResult.setPageNo(1);
        	queryResult.setData(Collections.<Consult>emptyList());
            return queryResult;
        }
        index = 1;
        boolean firstPage = (1 == pageNo);
        if(firstPage){
        	queryResult.setPageNo(1);
            sql = "SELECT CONSULT_ID,CONSULT_TITLE,CONSULTANT_ID,ASSESS_STAR,ASSESS_CONTANT,ASSESS_TIME,CREATE_TIME FROM CONSULT";
            sql = sql + whereSql;
            sql = sql + " ORDER BY ASSESS_TIME DESC ";
            sql = sql + " LIMIT " + pageSize;
        }else{
            int pageNumber = total / pageSize;
            if(total % pageSize != 0){
                ++pageNumber;
            }
            if(pageNo > pageNumber){
                pageNo = pageNumber;
            }
            queryResult.setPageNo(pageNo);
            --pageNo;
            int offset = (pageNo * pageSize);
            sql = "SELECT CONSULT_ID,CONSULT_TITLE,CONSULTANT_ID,ASSESS_STAR,ASSESS_CONTANT,ASSESS_TIME,CREATE_TIME FROM CONSULT";
            sql = sql + whereSql;
            sql = sql + " ORDER BY ASSESS_TIME DESC ";
            sql = sql + " LIMIT " + pageSize+ " OFFSET "+offset;
        }
        ps = con.prepareStatement(sql);
        try{
            ps.setString(index++,professorId);
            ResultSet rs = ps.executeQuery();
            try{
                List<Consult> consults = new ArrayList<Consult>();
                queryResult.setData(consults);
                int size = 0;
                while((size<pageSize) && rs.next()){
                    ++size;
                    Consult consult =  new Consult();
                    consult.setConsultId(rs.getString(1));
                    consult.setConsultTitle(rs.getString(2));
                    consult.setConsultantId(rs.getString(3));
                    consult.setAssessStar(rs.getInt(4));
                    consult.setAssessContant(rs.getString(5));
                    consult.setAssessTime(rs.getString(6));
                    consult.setCreateTime(rs.getString(7));
                    consults.add(consult);
                }
                return queryResult;
            }finally{
                try{rs.close();}catch(Exception e3){}
            }
        }finally{
            try{ps.close();}catch(Exception e4){}
        }
    }
	
	@SelectOne
	@Nullable
	public abstract Consult query(Connection con, String consultId) throws SQLException;
	
	@SelectList
	@OrderBy("ORDER BY CREATE_TIME DESC")
	public abstract List<Consult> query(Connection con) throws SQLException;
	
	/**
	 * 统计未查看的咨询条数
	 * @param con
	 * @param professorId 登陆者ID
	 * @return 返回未查看的资源条数
	 * @throws SQLException
	 */
	public int queryReadStatus(Connection con, String professorId) throws SQLException {
		int index = 1;
		String sql = " SELECT COUNT(1) FROM CONSULT WHERE READ_STATUS = 0 AND PROFESSOR_ID = ? ";
		PreparedStatement ps = con.prepareStatement(sql);
		try {
			ps.setString(index++, professorId);
			ResultSet rs = ps.executeQuery();
			try{
				rs.next();
				int count = rs.getInt(1);
				return count;
			}finally{
                try{rs.close();}catch(Exception e1){}
            }
		} finally{
            try{ps.close();}catch(Exception e2){}
        }
	}
	
	/**
	 * 统计所有未查看的消息记录（包括咨询别人和别人咨询自己的所有未读消息）
	 * @param con
	 * @param professorId 登陆者ID
	 * @return 返回所有未查看的消息条数
	 * @throws SQLException
	 */
	public int queryNotReadReply(Connection con, String professorId) throws SQLException {
		int index = 1;
		String sql = " SELECT COUNT(1) FROM TIDINGS "
				+ " INNER JOIN CONSULT ON TIDINGS.CONSULT_ID = CONSULT.CONSULT_ID AND (TIDINGS.READ_STATUS = 0) "
				+ " AND (CONSULT.CONSULTANT_ID = ? OR CONSULT.PROFESSOR_ID = ?) AND (TIDINGS.SENDER_ID != ?) ";
		PreparedStatement ps = con.prepareStatement(sql);
		try {
			ps.setString(index++, professorId);
			ps.setString(index++, professorId);
			ps.setString(index++, professorId);
			ResultSet rs = ps.executeQuery();
			try{
				rs.next();
				int count = rs.getInt(1);
				return count;
			}finally{
                try{rs.close();}catch(Exception e1){}
            }
		} finally{
            try{ps.close();}catch(Exception e2){}
        }
	}
	
	/**
	 * 我的工作台中显示接受咨询的总次数(已完成状态)
	 * @param con
	 * @param professorId 登陆者ID
	 * @return 返回接受咨询已完成的总次数
	 * @throws SQLException
	 */
	public int queryReceiveConsult(Connection con, String professorId) throws SQLException {
		int index = 1;
		String sql = " SELECT COUNT(1) FROM CONSULT WHERE (CONSULT_STATUS = 1) AND (PROFESSOR_ID = ?) ";
		PreparedStatement ps = con.prepareStatement(sql);
		try {
			ps.setString(index++, professorId);
			ResultSet rs = ps.executeQuery();
			try{
				rs.next();
				int count = rs.getInt(1);
				return count;
			}finally{
                try{rs.close();}catch(Exception e1){}
            }
		} finally{
            try{ps.close();}catch(Exception e2){}
        }
	}
	
	/**
	 * 我的工作台最新咨询
	 * @param con
	 * @param professorId 专家ID
	 * @return 返回最新未读咨询
	 * @throws SQLException
	 */
	public Consult queryNewConsult(Connection con, String professorId) throws SQLException {
		int index = 1;
		String sql = " SELECT CONSULT_ID,CONSULT_TYPE,CONSULT_TITLE,CONSULT_CONTANT,"
				+ "CONSULT.CREATE_TIME,CONSULT.PROFESSOR_ID,CONSULTANT_ID,READ_STATUS"
				+ " FROM CONSULT WHERE CONSULT.CREATE_TIME "
				+ " IN (SELECT MAX(CREATE_TIME) FROM CONSULT WHERE READ_STATUS = 0 AND PROFESSOR_ID = ?) ";
		PreparedStatement ps = con.prepareStatement(sql);
		try{
			ps.setString(index++, professorId);
			ResultSet rs = ps.executeQuery();
			try{
				if(rs.next()){
					Consult consult = new Consult();
					consult.setConsultId(rs.getString(1));
					consult.setConsultType(rs.getString(2));
					consult.setConsultTitle(rs.getString(3));
					consult.setConsultContant(rs.getString(4));
					consult.setCreateTime(rs.getString(5));
					consult.setProfessorId(rs.getString(6));
					consult.setConsultantId(rs.getString(7));
					consult.setReadStatus(rs.getInt(8));
					return consult;
				}else{
					return null;
				}
			}finally{
                try{rs.close();}catch(Exception e1){}
            }
        }finally{
            try{ps.close();}catch(Exception e2){}
        }
	}
	
	public Consult queryPro(Connection con,String consultId) throws SQLException{
        int index = 1;
        String sql = "SELECT CONSULT_ID,CONSULT_TYPE,CONSULT_TITLE,CONSULT_CONTANT,PROFESSOR_ID,CONSULTANT_ID,"
        		+ " CONSULT_STATUS,FINISH_TIME,ASSESS_STATUS,ASSESS_STAR,ASSESS_CONTANT,ASSESS_TIME,THANKS_STATUS,"
        		+ " THANKS_MONEY,THANKS_TIME,REVOVERY_TIME,CONSULT.READ_STATUS,CONSULT.CREATE_TIME,PROFESSOR.NAME "
        		+ " FROM CONSULT LEFT JOIN PROFESSOR ON CONSULT.CONSULTANT_ID = PROFESSOR.ID WHERE CONSULT_ID = ?";
        PreparedStatement ps = con.prepareStatement(sql);
        try{
            ps.setString(index++,consultId);
            ResultSet rs = ps.executeQuery();
            try{
                if(rs.next()){
                    Consult consult = new Consult();
                    consult.setConsultId(rs.getString(1));
                    consult.setConsultType(rs.getString(2));
                    consult.setConsultTitle(rs.getString(3));
                    consult.setConsultContant(rs.getString(4));
                    consult.setProfessorId(rs.getString(5));
                    EditProfessor professor = new EditProfessor();
                    professor.setId(rs.getString(6));
                    consult.setConsultStatus(rs.getInt(7));
                    String finishTime = rs.getString(8);
                    if(rs.wasNull()){
                    	finishTime = null;
                    }
                    consult.setFinishTime(finishTime);
                    consult.setAssessStatus(rs.getInt(9));
                    consult.setAssessStar(rs.getInt(10));
                    String assessContant = rs.getString(11);
                    if(rs.wasNull()){
                    	assessContant = null;
                    }
                    consult.setAssessContant(assessContant);
                    String assessTime = rs.getString(12);
                    if(rs.wasNull()){
                    	assessTime = null;
                    }
                    consult.setAssessTime(assessTime);
                    consult.setThanksStatus(rs.getInt(13));
                    BigDecimal thanksMoney = rs.getBigDecimal(14);
                    if(rs.wasNull()){
                    	thanksMoney = null;
                    }
                    consult.setThanksMoney(thanksMoney);
                    String thanksTime = rs.getString(15);
                    if(rs.wasNull()){
                    	thanksTime = null;
                    }
                    consult.setThanksTime(thanksTime);
                    consult.setRevoveryTime(rs.getString(16));
                    consult.setReadStatus(rs.getInt(17));
                    consult.setCreateTime(rs.getString(18));
                    professor.setName(rs.getString(19));
                    consult.setProfessor(professor);
                    return consult;
                }else{
                    return null;
                }
            }finally{
                try{rs.close();}catch(Exception e1){}
            }
        }finally{
            try{ps.close();}catch(Exception e2){}
        }
    }
	
	public Consult queryCon(Connection con,String consultId) throws SQLException{
        int index = 1;
        String sql = "SELECT CONSULT_ID,CONSULT_TYPE,CONSULT_TITLE,CONSULT_CONTANT,PROFESSOR_ID,CONSULTANT_ID,"
        		+ " CONSULT_STATUS,FINISH_TIME,ASSESS_STATUS,ASSESS_STAR,ASSESS_CONTANT,ASSESS_TIME,THANKS_STATUS,"
        		+ " THANKS_MONEY,THANKS_TIME,REVOVERY_TIME,CONSULT.READ_STATUS,CONSULT.CREATE_TIME,PROFESSOR.NAME "
        		+ " FROM CONSULT LEFT JOIN PROFESSOR ON CONSULT.PROFESSOR_ID = PROFESSOR.ID WHERE CONSULT_ID = ?";
        PreparedStatement ps = con.prepareStatement(sql);
        try{
            ps.setString(index++,consultId);
            ResultSet rs = ps.executeQuery();
            try{
                if(rs.next()){
                    Consult consult = new Consult();
                    consult.setConsultId(rs.getString(1));
                    consult.setConsultType(rs.getString(2));
                    consult.setConsultTitle(rs.getString(3));
                    consult.setConsultContant(rs.getString(4));
                    EditProfessor professor = new EditProfessor();
                    professor.setId(rs.getString(5));
                    consult.setConsultantId(rs.getString(6));
                    consult.setConsultStatus(rs.getInt(7));
                    String finishTime = rs.getString(8);
                    if(rs.wasNull()){
                    	finishTime = null;
                    }
                    consult.setFinishTime(finishTime);
                    consult.setAssessStatus(rs.getInt(9));
                    consult.setAssessStar(rs.getInt(10));
                    String assessContant = rs.getString(11);
                    if(rs.wasNull()){
                    	assessContant = null;
                    }
                    consult.setAssessContant(assessContant);
                    String assessTime = rs.getString(12);
                    if(rs.wasNull()){
                    	assessTime = null;
                    }
                    consult.setAssessTime(assessTime);
                    consult.setThanksStatus(rs.getInt(13));
                    BigDecimal thanksMoney = rs.getBigDecimal(14);
                    if(rs.wasNull()){
                    	thanksMoney = null;
                    }
                    consult.setThanksMoney(thanksMoney);
                    String thanksTime = rs.getString(15);
                    if(rs.wasNull()){
                    	thanksTime = null;
                    }
                    consult.setThanksTime(thanksTime);
                    consult.setRevoveryTime(rs.getString(16));
                    consult.setReadStatus(rs.getInt(17));
                    consult.setCreateTime(rs.getString(18));
                    professor.setName(rs.getString(19));
                    consult.setProfessor(professor);
                    return consult;
                }else{
                    return null;
                }
            }finally{
                try{rs.close();}catch(Exception e1){}
            }
        }finally{
            try{ps.close();}catch(Exception e2){}
        }
    }	
	
	public PageQueryResult<Consult> queryPageProBase(Connection con, String professorId, int status, 
			int timeType, int sortType, int pageSize, int pageNo) throws SQLException{
		PageQueryResult<Consult> queryResult = new PageQueryResult<Consult>();
		StringBuilder sql = new StringBuilder();
		sql.append(" WHERE PROFESSOR_ID = ? ");
		if(1 == status) {
			sql.append(" AND CONSULT_STATUS = 2 ");
		}else if(2 == status) {
			sql.append(" AND CONSULT_STATUS = 0 ");
		}else if(3 == status) {
			sql.append(" AND CONSULT_STATUS = 3 ");
		}else if(4 == status) {
			sql.append(" AND CONSULT_STATUS = 1 ");
		}else if(5 == status) {
			sql.append(" AND CONSULT_STATUS = 1 AND ASSESS_STATUS = 0 ");
		}
		StringBuilder whereSql = sql;
		sql = new StringBuilder();
		sql.append("SELECT COUNT(1) FROM CONSULT");
		if (whereSql.length() > 0) {
			sql.append(whereSql);
		}
		PreparedStatement ps = con.prepareStatement(sql.toString());
		int paramIndex = 1;
		int totalSize = 0;
		try {
			ps.setString(paramIndex++, professorId);
			queryResult.setPageSize(pageSize);
			ResultSet resultSet = ps.executeQuery();
			try {
				resultSet.next();
				totalSize = resultSet.getInt(1);
			} finally {
				try {
					resultSet.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
		queryResult.setTotal(totalSize);
		if (0 == totalSize) {
			queryResult.setPageNo(1);
			queryResult.setData(Collections.<Consult> emptyList());
			return queryResult;
		}
		paramIndex = 1;
		if (1 == pageNo) {
			queryResult.setPageNo(1);
			sql = new StringBuilder();
			sql.append(" SELECT CONSULT_ID,CONSULT_TYPE,CONSULT_TITLE,CONSULT_CONTANT,PROFESSOR_ID,CONSULTANT_ID,"
					+ " CONSULT_STATUS,FINISH_TIME,ASSESS_STATUS,ASSESS_STAR,ASSESS_CONTANT,ASSESS_TIME,THANKS_STATUS,"
					+ " THANKS_MONEY,THANKS_TIME,REVOVERY_TIME,READ_STATUS,CREATE_TIME FROM CONSULT ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			if(0 == timeType) {
				sql.append(" ORDER BY CREATE_TIME ");
			}else if(1 == timeType) {
				sql.append(" ORDER BY REVOVERY_TIME ");
			}else if(2 == timeType) {
				sql.append(" ORDER BY FINISH_TIME ");
			}
			if(0 == sortType) {
				sql.append(" DESC ");
			}
			sql.append(" LIMIT ").append(pageSize);
		} else {
			int _pageSize = totalSize / pageSize;
			if (totalSize % pageSize != 0) {
				++_pageSize;
			}
			if (pageNo > _pageSize) {
				pageNo = _pageSize;
			}
			queryResult.setPageNo(pageNo);
			--pageNo;
			int offset = (pageNo * pageSize);
			sql = new StringBuilder();
			sql.append(" SELECT CONSULT_ID,CONSULT_TYPE,CONSULT_TITLE,CONSULT_CONTANT,PROFESSOR_ID,CONSULTANT_ID,"
					+ " CONSULT_STATUS,FINISH_TIME,ASSESS_STATUS,ASSESS_STAR,ASSESS_CONTANT,ASSESS_TIME,THANKS_STATUS,"
					+ " THANKS_MONEY,THANKS_TIME,REVOVERY_TIME,READ_STATUS,CREATE_TIME FROM CONSULT ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			if(0 == timeType) {
				sql.append(" ORDER BY CREATE_TIME ");
			}else if(1 == timeType) {
				sql.append(" ORDER BY REVOVERY_TIME ");
			}else if(2 == timeType) {
				sql.append(" ORDER BY FINISH_TIME ");
			}
			if(0 == sortType) {
				sql.append(" DESC ");
			}
			sql.append(" LIMIT ").append(pageSize).append(" OFFSET ").append(offset);
		}
		ps = con.prepareStatement(sql.toString());
		try {
			ps.setString(paramIndex++, professorId);
			ResultSet rs = ps.executeQuery();
			try {
				List<Consult> consults = new ArrayList<Consult>();
				queryResult.setData(consults);
				while (rs.next()) {
					Consult consult = new Consult();
					consult.setConsultId(rs.getString(1));
					consult.setConsultType(rs.getString(2));
					consult.setConsultTitle(rs.getString(3));
					consult.setConsultContant(rs.getString(4));
					consult.setProfessorId(rs.getString(5));
					consult.setConsultantId(rs.getString(6));
					Integer consultStatus = rs.getInt(7);
					if (rs.wasNull()) {
						consultStatus = null;
					}
					consult.setConsultStatus(consultStatus);
					String finishTime = rs.getString(8);
					if (rs.wasNull()) {
						finishTime = null;
					}
					consult.setFinishTime(finishTime);
					Integer assessStatus = rs.getInt(9);
					if (rs.wasNull()) {
						assessStatus = null;
					}
					consult.setAssessStatus(assessStatus);
					consult.setAssessStar(rs.getInt(10));
					String assessContant = rs.getString(11);
					if (rs.wasNull()) {
						assessContant = null;
					}
					consult.setAssessContant(assessContant);
					String assessTime = rs.getString(12);
					if (rs.wasNull()) {
						assessTime = null;
					}
					consult.setAssessTime(assessTime);
					Integer thanksStatus = rs.getInt(13);
					if (rs.wasNull()) {
						thanksStatus = null;
					}
					consult.setThanksStatus(thanksStatus);
					BigDecimal thanksMoney = rs.getBigDecimal(14);
					if (rs.wasNull()) {
						thanksMoney = null;
					}
					consult.setThanksMoney(thanksMoney);
					String thanksTime = rs.getString(15);
					if (rs.wasNull()) {
						thanksTime = null;
					}
					consult.setThanksTime(thanksTime);
					consult.setRevoveryTime(rs.getString(16));
					consult.setReadStatus(rs.getInt(17));
					consult.setCreateTime(rs.getString(18));
					consults.add(consult);
				}
				return queryResult;
			} finally {
				try {
					rs.close();
				} catch (Exception e3) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e4) {
			}
		}
	}
	
	public PageQueryResult<Consult> queryPagePro(Connection con, String professorId, int status, int timeType, int sortType, int pageSize, int pageNo) throws SQLException{
		PageQueryResult<Consult> queryResult = new PageQueryResult<Consult>();
		StringBuilder sql = new StringBuilder();
		sql.append(" WHERE CONSULT.PROFESSOR_ID = ? ");
		if(1 == status) {
			sql.append(" AND CONSULT_STATUS = 2 ");
		}else if(2 == status) {
			sql.append(" AND CONSULT_STATUS = 0 ");
		}else if(3 == status) {
			sql.append(" AND CONSULT_STATUS = 3 ");
		}else if(4 == status) {
			sql.append(" AND CONSULT_STATUS = 1 ");
		}else if(5 == status) {
			sql.append(" AND CONSULT_STATUS = 1 AND ASSESS_STATUS = 0 ");
		}
		StringBuilder whereSql = sql;
		sql = new StringBuilder();
		sql.append("SELECT COUNT(1) FROM CONSULT");
		if (whereSql.length() > 0) {
			sql.append(whereSql);
		}
		PreparedStatement ps = con.prepareStatement(sql.toString());
		int paramIndex = 1;
		int totalSize = 0;
		try {
			ps.setString(paramIndex++, professorId);
			queryResult.setPageSize(pageSize);
			ResultSet resultSet = ps.executeQuery();
			try {
				resultSet.next();
				totalSize = resultSet.getInt(1);
			} finally {
				try {
					resultSet.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
		queryResult.setTotal(totalSize);
		if (0 == totalSize) {
			queryResult.setPageNo(1);
			queryResult.setData(Collections.<Consult> emptyList());
			return queryResult;
		}
		paramIndex = 1;
		if (1 == pageNo) {
			queryResult.setPageNo(1);
			sql = new StringBuilder();
			sql.append(" SELECT CONSULT_ID,CONSULT_TYPE,CONSULT_TITLE,CONSULT_CONTANT,CONSULT.PROFESSOR_ID,CONSULTANT_ID,"
					+ " CONSULT_STATUS,FINISH_TIME,ASSESS_STATUS,ASSESS_STAR,ASSESS_CONTANT,ASSESS_TIME,THANKS_STATUS,"
					+ " THANKS_MONEY,THANKS_TIME,REVOVERY_TIME,READ_STATUS,CONSULT.CREATE_TIME,PROFESSOR.NAME,DEPARTMENT,"
					+ " PROFESSOR.TITLE,ADDRESS,OFFICE,AUTHENTICATION,AUTH_TYPE,AUTH_STATUS,P.ORG_AUTH,ORGANIZATION.NAME "
					+ " FROM CONSULT INNER JOIN PROFESSOR ON CONSULTANT_ID = PROFESSOR.ID "
					+ " LEFT JOIN ORGANIZATION ON PROFESSOR.ORG_ID = ORGANIZATION.ID ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			if(0 == timeType) {
				sql.append(" ORDER BY CONSULT.CREATE_TIME ");
			}else if(1 == timeType) {
				sql.append(" ORDER BY REVOVERY_TIME ");
			}else if(2 == timeType) {
				sql.append(" ORDER BY FINISH_TIME ");
			}
			if(0 == sortType) {
				sql.append(" DESC ");
			}
			sql.append(" LIMIT ").append(pageSize);
		} else {
			int _pageSize = totalSize / pageSize;
			if (totalSize % pageSize != 0) {
				++_pageSize;
			}
			if (pageNo > _pageSize) {
				pageNo = _pageSize;
			}
			queryResult.setPageNo(pageNo);
			--pageNo;
			int offset = (pageNo * pageSize);
			sql = new StringBuilder();
			sql.append(" SELECT CONSULT_ID,CONSULT_TYPE,CONSULT_TITLE,CONSULT_CONTANT,CONSULT.PROFESSOR_ID,CONSULTANT_ID,"
					+ " CONSULT_STATUS,FINISH_TIME,ASSESS_STATUS,ASSESS_STAR,ASSESS_CONTANT,ASSESS_TIME,THANKS_STATUS,"
					+ " THANKS_MONEY,THANKS_TIME,REVOVERY_TIME,READ_STATUS,CONSULT.CREATE_TIME,PROFESSOR.NAME,DEPARTMENT,"
					+ " PROFESSOR.TITLE,ADDRESS,OFFICE,AUTHENTICATION,AUTH_TYPE,AUTH_STATUS,P.ORG_AUTH,ORGANIZATION.NAME "
					+ " FROM CONSULT INNER JOIN PROFESSOR ON CONSULTANT_ID = PROFESSOR.ID "
					+ " LEFT JOIN ORGANIZATION ON PROFESSOR.ORG_ID = ORGANIZATION.ID ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			if(0 == timeType) {
				sql.append(" ORDER BY CONSULT.CREATE_TIME ");
			}else if(1 == timeType) {
				sql.append(" ORDER BY REVOVERY_TIME ");
			}else if(2 == timeType) {
				sql.append(" ORDER BY FINISH_TIME ");
			}
			if(0 == sortType) {
				sql.append(" DESC ");
			}
			sql.append(" LIMIT ").append(pageSize).append(" OFFSET ").append(offset);
		}
		ps = con.prepareStatement(sql.toString());
		try {
			ps.setString(paramIndex++, professorId);
			ResultSet rs = ps.executeQuery();
			try {
				List<Consult> consults = new ArrayList<Consult>();
				queryResult.setData(consults);
				while (rs.next()) {
					Consult consult = new Consult();
					consult.setConsultId(rs.getString(1));
					consult.setConsultType(rs.getString(2));
					consult.setConsultTitle(rs.getString(3));
					consult.setConsultContant(rs.getString(4));
					consult.setProfessorId(rs.getString(5));
					EditProfessor professor = new EditProfessor();
					professor.setId(rs.getString(6));
					Integer consultStatus = rs.getInt(7);
					if (rs.wasNull()) {
						consultStatus = null;
					}
					consult.setConsultStatus(consultStatus);
					String finishTime = rs.getString(8);
					if (rs.wasNull()) {
						finishTime = null;
					}
					consult.setFinishTime(finishTime);
					Integer assessStatus = rs.getInt(9);
					if (rs.wasNull()) {
						assessStatus = null;
					}
					consult.setAssessStatus(assessStatus);
					Integer assessStar = rs.getInt(10);
					if (rs.wasNull()) {
						assessStar = null;
					}
					consult.setAssessStar(assessStar);
					String assessContant = rs.getString(11);
					if (rs.wasNull()) {
						assessContant = null;
					}
					consult.setAssessContant(assessContant);
					String assessTime = rs.getString(12);
					if (rs.wasNull()) {
						assessTime = null;
					}
					consult.setAssessTime(assessTime);
					Integer thanksStatus = rs.getInt(13);
					if (rs.wasNull()) {
						thanksStatus = null;
					}
					consult.setThanksStatus(thanksStatus);
					BigDecimal thanksMoney = rs.getBigDecimal(14);
					if (rs.wasNull()) {
						thanksMoney = null;
					}
					consult.setThanksMoney(thanksMoney);
					String thanksTime = rs.getString(15);
					if (rs.wasNull()) {
						thanksTime = null;
					}
					consult.setThanksTime(thanksTime);
					consult.setRevoveryTime(rs.getString(16));
					consult.setReadStatus(rs.getInt(17));
					consult.setCreateTime(rs.getString(18));
					professor.setName(rs.getString(19));
					professor.setDepartment(rs.getString(20));
					professor.setTitle(rs.getString(21));
					String address = rs.getString(22);
					if (rs.wasNull()) {
						address = null;
					}
					professor.setAddress(address);
					professor.setOffice(rs.getString(23));
					professor.setAuthentication(rs.getInt(24));
					professor.setAuthType(rs.getInt(25));
					professor.setAuthStatus(rs.getInt(26));
					professor.setOrgAuth(rs.getString(27));
					String orgName = rs.getString(28);
					if(rs.wasNull()){
						orgName = null;
					}
					professor.setOrgName(orgName);
					consult.setProfessor(professor);
					consults.add(consult);
				}
				return queryResult;
			} finally {
				try {
					rs.close();
				} catch (Exception e3) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e4) {
			}
		}
	}
	
	public PageQueryResult<Consult> queryPageConBase(Connection con, String consultantId, int status, int timeType, int sortType, int pageSize, int pageNo) throws SQLException{
		PageQueryResult<Consult> queryResult = new PageQueryResult<Consult>();
		StringBuilder sql = new StringBuilder();
		sql.append(" WHERE CONSULTANT_ID = ? ");
		if(1 == status) {
			sql.append(" AND CONSULT_STATUS = 2 ");
		}else if(2 == status) {
			sql.append(" AND CONSULT_STATUS = 0 ");
		}else if(3 == status) {
			sql.append(" AND CONSULT_STATUS = 3 ");
		}else if(4 == status) {
			sql.append(" AND CONSULT_STATUS = 1 ");
		}else if(5 == status) {
			sql.append(" AND CONSULT_STATUS = 1 AND ASSESS_STATUS = 0 ");
		}
		StringBuilder whereSql = sql;
		sql = new StringBuilder();
		sql.append("SELECT COUNT(1) FROM CONSULT");
		if (whereSql.length() > 0) {
			sql.append(whereSql);
		}
		PreparedStatement ps = con.prepareStatement(sql.toString());
		int paramIndex = 1;
		int totalSize = 0;
		try {
			ps.setString(paramIndex++, consultantId);
			ResultSet resultSet = ps.executeQuery();
			try {
				resultSet.next();
				totalSize = resultSet.getInt(1);
			} finally {
				try {
					resultSet.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
		queryResult.setTotal(totalSize);
		if (0 == totalSize) {
			queryResult.setPageNo(1);
			queryResult.setData(Collections.<Consult> emptyList());
			return queryResult;
		}
		paramIndex = 1;
		if (1 == pageNo) {
			queryResult.setPageNo(1);
			sql = new StringBuilder();
			sql.append(" SELECT CONSULT_ID,CONSULT_TYPE,CONSULT_TITLE,CONSULT_CONTANT,PROFESSOR_ID,CONSULTANT_ID,"
					+ " CONSULT_STATUS,FINISH_TIME,ASSESS_STATUS,ASSESS_STAR,ASSESS_CONTANT,ASSESS_TIME,THANKS_STATUS,"
					+ " THANKS_MONEY,THANKS_TIME,REVOVERY_TIME,READ_STATUS,CREATE_TIME FROM CONSULT ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			if(0 == timeType) {
				sql.append(" ORDER BY CREATE_TIME ");
			}else if(1 == timeType) {
				sql.append(" ORDER BY REVOVERY_TIME ");
			}else if(2 == timeType) {
				sql.append(" ORDER BY FINISH_TIME ");
			}
			if(0 == sortType) {
				sql.append(" DESC ");
			}
			sql.append(" LIMIT ").append(pageSize);
		} else {
			int _pageSize = totalSize / pageSize;
			if (totalSize % pageSize != 0) {
				++_pageSize;
			}
			if (pageNo > _pageSize) {
				pageNo = _pageSize;
			}
			queryResult.setPageNo(pageNo);
			--pageNo;
			int offset = (pageNo * pageSize);
			sql = new StringBuilder();
			sql.append(" SELECT CONSULT_ID,CONSULT_TYPE,CONSULT_TITLE,CONSULT_CONTANT,PROFESSOR_ID,CONSULTANT_ID,"
					+ " CONSULT_STATUS,FINISH_TIME,ASSESS_STATUS,ASSESS_STAR,ASSESS_CONTANT,ASSESS_TIME,THANKS_STATUS,"
					+ " THANKS_MONEY,THANKS_TIME,REVOVERY_TIME,READ_STATUS,CREATE_TIME FROM CONSULT ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			if(0 == timeType) {
				sql.append(" ORDER BY CREATE_TIME ");
			}else if(1 == timeType) {
				sql.append(" ORDER BY REVOVERY_TIME ");
			}else if(2 == timeType) {
				sql.append(" ORDER BY FINISH_TIME ");
			}
			if(0 == sortType) {
				sql.append(" DESC ");
			}
			sql.append(" LIMIT ").append(pageSize).append(" OFFSET ").append(offset);
		}
		ps = con.prepareStatement(sql.toString());
		try {
			ps.setString(paramIndex++, consultantId);
			ResultSet rs = ps.executeQuery();
			try {
				List<Consult> consults = new ArrayList<Consult>();
				queryResult.setData(consults);
				while (rs.next()) {
					Consult consult = new Consult();
					consult.setConsultId(rs.getString(1));
					consult.setConsultType(rs.getString(2));
					consult.setConsultTitle(rs.getString(3));
					consult.setConsultContant(rs.getString(4));
					consult.setProfessorId(rs.getString(5));
					consult.setConsultantId(rs.getString(6));
					Integer consultStatus = rs.getInt(7);
					if (rs.wasNull()) {
						consultStatus = null;
					}
					consult.setConsultStatus(consultStatus);
					String finishTime = rs.getString(8);
					if (rs.wasNull()) {
						finishTime = null;
					}
					consult.setFinishTime(finishTime);
					Integer assessStatus = rs.getInt(9);
					if (rs.wasNull()) {
						assessStatus = null;
					}
					consult.setAssessStatus(assessStatus);
					consult.setAssessStar(rs.getInt(10));
					String assessContant = rs.getString(11);
					if (rs.wasNull()) {
						assessContant = null;
					}
					consult.setAssessContant(assessContant);
					String assessTime = rs.getString(12);
					if (rs.wasNull()) {
						assessTime = null;
					}
					consult.setAssessTime(assessTime);
					Integer thanksStatus = rs.getInt(13);
					if (rs.wasNull()) {
						thanksStatus = null;
					}
					consult.setThanksStatus(thanksStatus);
					BigDecimal thanksMoney = rs.getBigDecimal(14);
					if (rs.wasNull()) {
						thanksMoney = null;
					}
					consult.setThanksMoney(thanksMoney);
					String thanksTime = rs.getString(15);
					if (rs.wasNull()) {
						thanksTime = null;
					}
					consult.setThanksTime(thanksTime);
					consult.setRevoveryTime(rs.getString(16));
					consult.setReadStatus(rs.getInt(17));
					consult.setCreateTime(rs.getString(18));
					consults.add(consult);
				}
				return queryResult;
			} finally {
				try {
					rs.close();
				} catch (Exception e3) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e4) {
			}
		}
	}
	
	public PageQueryResult<Consult> queryPageCon(Connection con, String consultantId, int status, int timeType, int sortType, int pageSize, int pageNo) throws SQLException{
		PageQueryResult<Consult> queryResult = new PageQueryResult<Consult>();
		StringBuilder sql = new StringBuilder();
		sql.append(" WHERE CONSULT.CONSULTANT_ID = ? ");
		if(1 == status) {
			sql.append(" AND CONSULT_STATUS = 2 ");
		}else if(2 == status) {
			sql.append(" AND CONSULT_STATUS = 0 ");
		}else if(3 == status) {
			sql.append(" AND CONSULT_STATUS = 3 ");
		}else if(4 == status) {
			sql.append(" AND CONSULT_STATUS = 1 ");
		}else if(5 == status) {
			sql.append(" AND CONSULT_STATUS = 1 AND ASSESS_STATUS = 0 ");
		}
		StringBuilder whereSql = sql;
		sql = new StringBuilder();
		sql.append("SELECT COUNT(1) FROM CONSULT");
		if (whereSql.length() > 0) {
			sql.append(whereSql);
		}
		PreparedStatement ps = con.prepareStatement(sql.toString());
		int paramIndex = 1;
		int totalSize = 0;
		try {
			ps.setString(paramIndex++, consultantId);
			ResultSet resultSet = ps.executeQuery();
			try {
				resultSet.next();
				totalSize = resultSet.getInt(1);
			} finally {
				try {
					resultSet.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
		queryResult.setTotal(totalSize);
		if (0 == totalSize) {
			queryResult.setPageNo(1);
			queryResult.setData(Collections.<Consult> emptyList());
			return queryResult;
		}
		paramIndex = 1;
		if (1 == pageNo) {
			queryResult.setPageNo(1);
			sql = new StringBuilder();
			sql.append(" SELECT CONSULT_ID,CONSULT_TYPE,CONSULT_TITLE,CONSULT_CONTANT,CONSULT.PROFESSOR_ID,CONSULTANT_ID,"
					+ " CONSULT_STATUS,FINISH_TIME,ASSESS_STATUS,ASSESS_STAR,ASSESS_CONTANT,ASSESS_TIME,THANKS_STATUS,"
					+ " THANKS_MONEY,THANKS_TIME,REVOVERY_TIME,CONSULT.READ_STATUS,CONSULT.CREATE_TIME,PROFESSOR.NAME,"
					+ " DEPARTMENT,TITLE,ADDRESS,OFFICE,AUTHENTICATION,AUTH_TYPE,AUTH_STATUS,P.ORG_AUTH,ORGANIZATION.NAME "
					+ " FROM CONSULT INNER JOIN PROFESSOR ON PROFESSOR_ID = PROFESSOR.ID "
					+ " LEFT JOIN ORGANIZATION ON PROFESSOR.ORG_ID = ORGANIZATION.ID ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			if(0 == timeType) {
				sql.append(" ORDER BY CONSULT.CREATE_TIME ");
			}else if(1 == timeType) {
				sql.append(" ORDER BY REVOVERY_TIME ");
			}else if(2 == timeType) {
				sql.append(" ORDER BY FINISH_TIME ");
			}
			if(0 == sortType) {
				sql.append(" DESC ");
			}
			sql.append(" LIMIT ").append(pageSize);
		} else {
			int _pageSize = totalSize / pageSize;
			if (totalSize % pageSize != 0) {
				++_pageSize;
			}
			if (pageNo > _pageSize) {
				pageNo = _pageSize;
			}
			queryResult.setPageNo(pageNo);
			--pageNo;
			int offset = (pageNo * pageSize);
			sql = new StringBuilder();
			sql.append(" SELECT CONSULT_ID,CONSULT_TYPE,CONSULT_TITLE,CONSULT_CONTANT,CONSULT.PROFESSOR_ID,CONSULTANT_ID,"
					+ " CONSULT_STATUS,FINISH_TIME,ASSESS_STATUS,ASSESS_STAR,ASSESS_CONTANT,ASSESS_TIME,THANKS_STATUS,"
					+ " THANKS_MONEY,THANKS_TIME,REVOVERY_TIME,CONSULT.READ_STATUS,CONSULT.CREATE_TIME,PROFESSOR.NAME,"
					+ " DEPARTMENT,TITLE,ADDRESS,OFFICE,AUTHENTICATION,AUTH_TYPE,AUTH_STATUS,P.ORG_AUTH,ORGANIZATION.NAME "
					+ " FROM CONSULT INNER JOIN PROFESSOR ON PROFESSOR_ID = PROFESSOR.ID "
					+ " LEFT JOIN ORGANIZATION ON PROFESSOR.ORG_ID = ORGANIZATION.ID ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			if(0 == timeType) {
				sql.append(" ORDER BY CONSULT.CREATE_TIME ");
			}else if(1 == timeType) {
				sql.append(" ORDER BY REVOVERY_TIME ");
			}else if(2 == timeType) {
				sql.append(" ORDER BY FINISH_TIME ");
			}
			if(0 == sortType) {
				sql.append(" DESC ");
			}
			sql.append(" LIMIT ").append(pageSize).append(" OFFSET ").append(offset);
		}
		ps = con.prepareStatement(sql.toString());
		try {
			ps.setString(paramIndex++, consultantId);
			ResultSet rs = ps.executeQuery();
			try {
				List<Consult> consults = new ArrayList<Consult>();
				queryResult.setData(consults);
				while (rs.next()) {
					Consult consult = new Consult();
					consult.setConsultId(rs.getString(1));
					consult.setConsultType(rs.getString(2));
					consult.setConsultTitle(rs.getString(3));
					consult.setConsultContant(rs.getString(4));
					EditProfessor professor = new EditProfessor();
					professor.setId(rs.getString(5));
					consult.setConsultantId(rs.getString(6));
					Integer consultStatus = rs.getInt(7);
					if (rs.wasNull()) {
						consultStatus = null;
					}
					consult.setConsultStatus(consultStatus);
					String finishTime = rs.getString(8);
					if (rs.wasNull()) {
						finishTime = null;
					}
					consult.setFinishTime(finishTime);
					Integer assessStatus = rs.getInt(9);
					if (rs.wasNull()) {
						assessStatus = null;
					}
					consult.setAssessStatus(assessStatus);
					consult.setAssessStar(rs.getInt(10));
					String assessContant = rs.getString(11);
					if (rs.wasNull()) {
						assessContant = null;
					}
					consult.setAssessContant(assessContant);
					String assessTime = rs.getString(12);
					if (rs.wasNull()) {
						assessTime = null;
					}
					consult.setAssessTime(assessTime);
					Integer thanksStatus = rs.getInt(13);
					if (rs.wasNull()) {
						thanksStatus = null;
					}
					consult.setThanksStatus(thanksStatus);
					BigDecimal thanksMoney = rs.getBigDecimal(14);
					if (rs.wasNull()) {
						thanksMoney = null;
					}
					consult.setThanksMoney(thanksMoney);
					String thanksTime = rs.getString(15);
					if (rs.wasNull()) {
						thanksTime = null;
					}
					consult.setThanksTime(thanksTime);
					consult.setRevoveryTime(rs.getString(16));
					consult.setReadStatus(rs.getInt(17));
					consult.setCreateTime(rs.getString(18));
					professor.setName(rs.getString(19));
					professor.setDepartment(rs.getString(20));
					professor.setTitle(rs.getString(21));
					String address = rs.getString(22);
					if (rs.wasNull()) {
						address = null;
					}
					professor.setAddress(address);
					professor.setOffice(rs.getString(23));
					professor.setAuthentication(rs.getInt(24));
					professor.setAuthType(rs.getInt(25));
					professor.setAuthStatus(rs.getInt(26));
					professor.setOrgAuth(rs.getString(27));
					String orgName = rs.getString(28);
					if(rs.wasNull()){
						orgName = null;
					}
					professor.setOrgName(orgName);
					consult.setProfessor(professor);
					consults.add(consult);
				}
				return queryResult;
			} finally {
				try {
					rs.close();
				} catch (Exception e3) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e4) {
			}
		}
	}
	
	/**
	 * 按条件搜索所有咨询的接口
	 * @param con
	 * @param professorId 用户ID
	 * @param consultOrNeed 接受咨询或咨询别人的状态值，1-收到咨询，2-我的需求,默认为1
	 * @param consultType 咨询类型，包括技术咨询、资源咨询、其他事务
	 * @param status 查询状态 0-全部，1-待回复/新咨询，2-进行中，3-已谢绝，4-已完成，5-待评价，默认为0
	 * @param timeType 排序类型 0-按发起时间，1-按最后回复时间，2-按完成时间，默认为0
	 * @param pageSize
	 * @param pageNo
	 * @return 返回咨询列表
	 * @throws SQLException
	 */
	public PageQueryResult<Consult> queryPageBase(Connection con, String professorId, int consultOrNeed, 
			String consultType, int status, int timeType, int pageSize, int pageNo) throws SQLException{
		PageQueryResult<Consult> queryResult = new PageQueryResult<Consult>();
		StringBuilder sql = new StringBuilder();
		if(0 == consultOrNeed){
			sql.append(" WHERE (PROFESSOR_ID = ? OR CONSULTANT_ID = ?) ");
		}else if(1 == consultOrNeed){
			sql.append(" WHERE PROFESSOR_ID = ? ");
		}else if(2 == consultOrNeed){
			sql.append(" WHERE CONSULTANT_ID = ? ");
		}
		if(consultType != null){
			if("技术咨询".equals(consultType)){
				sql.append(" AND CONSULT_TYPE = '技术咨询' ");
			}else if("资源咨询".equals(consultType)){
				sql.append(" AND CONSULT_TYPE = '资源咨询' ");
			}else if("其他事务".equals(consultType)){
				sql.append(" AND CONSULT_TYPE = '其他事务' ");
			}
		}
		if(1 == status) {
			sql.append(" AND CONSULT_STATUS = 2 ");
		}else if(2 == status) {
			sql.append(" AND CONSULT_STATUS = 0 ");
		}else if(3 == status) {
			sql.append(" AND CONSULT_STATUS = 3 ");
		}else if(4 == status) {
			sql.append(" AND CONSULT_STATUS = 1 ");
		}else if(5 == status) {
			sql.append(" AND CONSULT_STATUS = 1 AND ASSESS_STATUS = 0 ");
		}
		StringBuilder whereSql = sql;
		sql = new StringBuilder();
		sql.append("SELECT COUNT(1) FROM CONSULT");
		if (whereSql.length() > 0) {
			sql.append(whereSql);
		}
		PreparedStatement ps = con.prepareStatement(sql.toString());
		int paramIndex = 1;
		int totalSize = 0;
		try {
			ps.setString(paramIndex++, professorId);
			if(0 == consultOrNeed){
				ps.setString(paramIndex++, professorId);
			}
			queryResult.setPageSize(pageSize);
			ResultSet resultSet = ps.executeQuery();
			try {
				resultSet.next();
				totalSize = resultSet.getInt(1);
			} finally {
				try {
					resultSet.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
		queryResult.setTotal(totalSize);
		if (0 == totalSize) {
			queryResult.setPageNo(1);
			queryResult.setData(Collections.<Consult> emptyList());
			return queryResult;
		}
		paramIndex = 1;
		if (1 == pageNo) {
			queryResult.setPageNo(1);
			sql = new StringBuilder();
			sql.append(" SELECT CONSULT_ID,CONSULT_TYPE,CONSULT_TITLE,CONSULT_CONTANT,PROFESSOR_ID,CONSULTANT_ID,"
					+ " CONSULT_STATUS,FINISH_TIME,ASSESS_STATUS,ASSESS_STAR,ASSESS_CONTANT,ASSESS_TIME,THANKS_STATUS,"
					+ " THANKS_MONEY,THANKS_TIME,REVOVERY_TIME,READ_STATUS,CREATE_TIME FROM CONSULT ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			if(0 == timeType) {
				sql.append(" ORDER BY CREATE_TIME ");
			}else if(1 == timeType) {
				sql.append(" ORDER BY REVOVERY_TIME DESC ");
			}else if(2 == timeType) {
				sql.append(" ORDER BY FINISH_TIME DESC ");
			}
			sql.append(" LIMIT ").append(pageSize);
		} else {
			int _pageSize = totalSize / pageSize;
			if (totalSize % pageSize != 0) {
				++_pageSize;
			}
			if (pageNo > _pageSize) {
				pageNo = _pageSize;
			}
			queryResult.setPageNo(pageNo);
			--pageNo;
			int offset = (pageNo * pageSize);
			sql = new StringBuilder();
			sql.append(" SELECT CONSULT_ID,CONSULT_TYPE,CONSULT_TITLE,CONSULT_CONTANT,PROFESSOR_ID,CONSULTANT_ID,"
					+ " CONSULT_STATUS,FINISH_TIME,ASSESS_STATUS,ASSESS_STAR,ASSESS_CONTANT,ASSESS_TIME,THANKS_STATUS,"
					+ " THANKS_MONEY,THANKS_TIME,REVOVERY_TIME,READ_STATUS,CREATE_TIME FROM CONSULT ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			if(0 == timeType) {
				sql.append(" ORDER BY CREATE_TIME ");
			}else if(1 == timeType) {
				sql.append(" ORDER BY REVOVERY_TIME DESC ");
			}else if(2 == timeType) {
				sql.append(" ORDER BY FINISH_TIME DESC ");
			}
			sql.append(" LIMIT ").append(pageSize).append(" OFFSET ").append(offset);
		}
		ps = con.prepareStatement(sql.toString());
		try {
			ps.setString(paramIndex++, professorId);
			if(0 == consultOrNeed){
				ps.setString(paramIndex++, professorId);
			}
			ResultSet rs = ps.executeQuery();
			try {
				List<Consult> consults = new ArrayList<Consult>();
				queryResult.setData(consults);
				while (rs.next()) {
					Consult consult = new Consult();
					consult.setConsultId(rs.getString(1));
					consult.setConsultType(rs.getString(2));
					consult.setConsultTitle(rs.getString(3));
					consult.setConsultContant(rs.getString(4));
					consult.setProfessorId(rs.getString(5));
					consult.setConsultantId(rs.getString(6));
					Integer consultStatus = rs.getInt(7);
					if (rs.wasNull()) {
						consultStatus = null;
					}
					consult.setConsultStatus(consultStatus);
					String finishTime = rs.getString(8);
					if (rs.wasNull()) {
						finishTime = null;
					}
					consult.setFinishTime(finishTime);
					Integer assessStatus = rs.getInt(9);
					if (rs.wasNull()) {
						assessStatus = null;
					}
					consult.setAssessStatus(assessStatus);
					consult.setAssessStar(rs.getInt(10));
					String assessContant = rs.getString(11);
					if (rs.wasNull()) {
						assessContant = null;
					}
					consult.setAssessContant(assessContant);
					String assessTime = rs.getString(12);
					if (rs.wasNull()) {
						assessTime = null;
					}
					consult.setAssessTime(assessTime);
					Integer thanksStatus = rs.getInt(13);
					if (rs.wasNull()) {
						thanksStatus = null;
					}
					consult.setThanksStatus(thanksStatus);
					BigDecimal thanksMoney = rs.getBigDecimal(14);
					if (rs.wasNull()) {
						thanksMoney = null;
					}
					consult.setThanksMoney(thanksMoney);
					String thanksTime = rs.getString(15);
					if (rs.wasNull()) {
						thanksTime = null;
					}
					consult.setThanksTime(thanksTime);
					consult.setRevoveryTime(rs.getString(16));
					consult.setReadStatus(rs.getInt(17));
					consult.setCreateTime(rs.getString(18));
					consults.add(consult);
				}
				return queryResult;
			} finally {
				try {
					rs.close();
				} catch (Exception e3) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e4) {
			}
		}
	}
	
}
