package com.ekexiu.portal.oauth;

import java.io.IOException;
import java.lang.reflect.Type;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.method.SetCookie;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.util.context.JfwAppContext;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.reflect.TypeReference;
import org.jfw.util.state.StateCode;

import com.ekexiu.portal.dao.ProfessorDao;
import com.ekexiu.portal.dao.UserDao;
import com.ekexiu.portal.dao.UserOpenIdDao;
import com.ekexiu.portal.po.Professor;
import com.ekexiu.portal.po.User;
import com.ekexiu.portal.po.UserOpenId;
import com.ekexiu.portal.pojo.SessionUser;
import com.ekexiu.portal.util.HttpUtil;

@Path("/oauth")
public class OAuthService {

	private static final Type MAP_TYPE = new TypeReference<Map<String, Object>>() {
	}.getType();

	private long expriesWithOpenId = 1000 * 60 * 10;

	@Autowrie
	private UserOpenIdDao userOpenIdDao;
	@Autowrie
	private UserDao userDao;
	@Autowrie
	private ProfessorDao professorDao;

	private String xcxAppid = "wx92750263b1138a96";
	private String xcxSecret = "246e17b10923a30bfd6cc8da1ca88b8a";

	private Map<String, OAuthHandler> handlers;

	public Map<String, OAuthHandler> getHandlers() {
		return handlers;
	}

	public void setHandlers(Map<String, OAuthHandler> handlers) {
		this.handlers = handlers;
	}

	public UserOpenIdDao getUserOpenIdDao() {
		return userOpenIdDao;
	}

	public void setUserOpenIdDao(UserOpenIdDao userOpenIdDao) {
		this.userOpenIdDao = userOpenIdDao;
	}

	public UserDao getUserDao() {
		return userDao;
	}

	public void setUserDao(UserDao userDao) {
		this.userDao = userDao;
	}

	public long getExpriesWithOpenId() {
		return expriesWithOpenId;
	}

	public void setExpriesWithOpenId(long expriesWithOpenId) {
		this.expriesWithOpenId = expriesWithOpenId;
	}

	public ProfessorDao getProfessorDao() {
		return professorDao;
	}

	public void setProfessorDao(ProfessorDao professorDao) {
		this.professorDao = professorDao;
	}

	@Get
	@Path("/validCode")
	public AuthLoginResponse validCode(@JdbcConn Connection con, String code, String state) throws SQLException, JfwBaseException {
		OAuthHandler oah = this.handlers.get(state);
		if (oah == null)
			throw new IllegalArgumentException("非法的参数{state=" + state);
		OAuthUser ou = oah.login(code);
		UserOpenId uoi = this.userOpenIdDao.queryByOpenid(con, oah.getType(), ou.getOpenId());

		StateCode<OAuthUser, UserOpenId> sc = new StateCode<OAuthUser, UserOpenId>();
		final String key = JfwAppContext.cacheObjectAndGenKey(sc);
		sc.setKey(ou);
		sc.setValue(uoi);
		sc.setBuildTime(System.currentTimeMillis());
		sc.setExpiredTime(sc.getBuildTime() + this.expriesWithOpenId);
		JfwAppContext.getScheduledExecutorService().schedule(new Runnable() {
			@Override
			public void run() {
				JfwAppContext.removeCachedObject(key);
			}
		}, this.expriesWithOpenId, TimeUnit.MILLISECONDS);
		AuthLoginResponse alr = new AuthLoginResponse();
		alr.setAssociated(null != uoi);
		alr.setAuthCode(key);
		return alr;
	}

	@SetCookie(checkResultNull = true, path = "/", value = { "userid=result.getId()", "userMobilePhone=result.getMobilePhone()", "userType=result.getType()",
			"userAuth=String.valueOf(result.isAuth())", "userEmail=result.getEmail()==null?\"\":result.getEmail()",
			"userName=result.getName()==null?\"\":java.net.URLEncoder.encode(result.getName(),\"utf-8\")" })
	@Post
	@Path("/login")
	public SessionUser login(@JdbcConn Connection con, String authCode) throws SQLException, JfwBaseException {
		@SuppressWarnings("unchecked")
		StateCode<OAuthUser, UserOpenId> sc = (StateCode<OAuthUser, UserOpenId>) JfwAppContext.getCachedObject(authCode);

		if (sc == null || sc.getExpiredTime() < System.currentTimeMillis())
			throw new JfwBaseException(-1, "authCode is expired");
		if (null == sc.getValue())
			throw new JfwBaseException(-2, "authCode is invalid openid not associate local user");

		User user = this.userDao.query(con, sc.getValue().getUserid());
		if (null == user)
			return null;

		JfwAppContext.removeCachedObject(authCode);
		SessionUser ret = new SessionUser();
		ret.setId(user.getId());
		Professor professor = this.professorDao.queryOne(con, user.getId());
		if (null != professor) {
			ret.setName(professor.getName());
		}
		ret.setMobilePhone(user.getMobilePhone());
		ret.setType(user.getUserType());
		ret.setEmail(user.getEmail());
		ret.setAuth(true);
		return ret;
	}

	@SetCookie(checkResultNull = true, path = "/", value = { "userid=result.getId()", "userMobilePhone=result.getMobilePhone()", "userType=result.getType()",
			"userAuth=String.valueOf(result.isAuth())", "userEmail=result.getEmail()==null?\"\":result.getEmail()",
			"userName=result.getName()==null?\"\":java.net.URLEncoder.encode(result.getName(),\"utf-8\")" })
	@Post
	@Path("/openidLogin")
	public SessionUser openidLogin(@JdbcConn Connection con, String openid, String oauthType) throws SQLException, JfwBaseException {
		if (null == this.handlers.get(oauthType)) {
			throw new JfwBaseException(-1, "invalid oauthType");
		}
		UserOpenId uoi = this.userOpenIdDao.queryByOpenid(con, oauthType, openid);
		if (uoi != null) {
			User user = this.userDao.query(con, uoi.getUserid());
			if (user != null) {
				SessionUser ret = new SessionUser();
				ret.setId(user.getId());
				Professor professor = this.professorDao.queryOne(con, user.getId());
				if (null != professor) {
					ret.setName(professor.getName());
				}
				ret.setMobilePhone(user.getMobilePhone());
				ret.setType(user.getUserType());
				ret.setEmail(user.getEmail());
				ret.setAuth(true);
				return ret;
			}
		}
		return null;
	}

	@Post
	@Path("/associate")
	public void associate(@JdbcConn(false) Connection con, String userid, String openid, String oauthType) throws SQLException, JfwBaseException {
		if (null == this.handlers.get(oauthType)) {
			throw new JfwBaseException(-1, "invalid oauthType");
		}
		UserOpenId uoi = new UserOpenId();
		uoi.setOauthType(oauthType);
		uoi.setUserid(userid);
		uoi.setOpenid(openid);
		try {
			userOpenIdDao.insert(con, uoi);
			con.commit();
		} catch (SQLException e) {
			if ("23505".equals(e.getSQLState())) {
				try {
					con.rollback();
				} catch (Throwable th) {
				}
				throw new JfwBaseException(-2, "duplicate associate");
			} else {
				throw e;
			}
		}
	}

	@Post
	@Path("/checkAssociated")
	public boolean associcated(@JdbcConn(false) Connection con, String userid, String oauthType) throws SQLException, JfwBaseException {
		if (null == this.handlers.get(oauthType)) {
			throw new JfwBaseException(-1, "invalid oauthType");
		}
		return userOpenIdDao.queryByUserid(con, oauthType, userid) != null;
	}

	@Post
	@Path("/validOpenid")
	public AuthLoginResponse validOpenid(@JdbcConn Connection con, String authType, String openid) throws SQLException, JfwBaseException {
		OAuthUser ou = new OAuthUser();
		ou.setType(authType);
		ou.setOpenId(openid);
		UserOpenId uoi = this.userOpenIdDao.queryByOpenid(con, authType, openid);
		AuthLoginResponse alr = new AuthLoginResponse();
		StateCode<OAuthUser, UserOpenId> sc = new StateCode<OAuthUser, UserOpenId>();
		final String key = JfwAppContext.cacheObjectAndGenKey(sc);
		sc.setKey(ou);
		sc.setValue(uoi);
		sc.setBuildTime(System.currentTimeMillis());
		sc.setExpiredTime(sc.getBuildTime() + this.expriesWithOpenId);
		JfwAppContext.getScheduledExecutorService().schedule(new Runnable() {
			@Override
			public void run() {
				JfwAppContext.removeCachedObject(key);
			}
		}, this.expriesWithOpenId, TimeUnit.MILLISECONDS);
		alr.setAssociated(null != uoi);
		alr.setAuthCode(key);
		return alr;
	}

	@Get
	@Path("/redirectUris")
	public Map<String, String> getRedirectUris() {
		Map<String, String> ret = new HashMap<String, String>();
		for (Map.Entry<String, OAuthHandler> h : this.handlers.entrySet()) {
			ret.put(h.getKey(), h.getValue().getLoginUrl());
		}
		return ret;
	}

	@Get
	@Path("/xcx/openid")
	public String jscode2session(String code) throws IOException {
		return (String) HttpUtil.<Map<String, Object>> loadJsonObject("https://api.weixin.qq.com/sns/jscode2session?appid=" + this.xcxAppid + "&secret="
				+ this.xcxSecret + "&js_code=" + code + "&grant_type=authorization_code", MAP_TYPE).get("openid");
	}

	public static class AuthLoginResponse {
		private boolean associated;
		private String authCode;

		public boolean isAssociated() {
			return associated;
		}

		public void setAssociated(boolean associated) {
			this.associated = associated;
		}

		public String getAuthCode() {
			return authCode;
		}

		public void setAuthCode(String authCode) {
			this.authCode = authCode;
		}
	}

}
