package com.ekexiu.portal.util;

import java.io.IOException;
import java.util.LinkedList;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.jfw.util.context.JfwAppContext;
import org.jfw.util.state.StateCode;

public class PictureVC extends HttpServlet implements Servlet {

	private static final long serialVersionUID = 2459158903781864570L;

	private static long timeout = 1000 * 60 * 5;
	private static long interval = 1000 * 60;

	private static ConcurrentHashMap<String, StateCode<String, String>> tokens = new ConcurrentHashMap<String, StateCode<String, String>>();

	public static void put(String key, String val) {
		StateCode<String, String> code = new StateCode<String, String>();
		code.setBuildTime(System.currentTimeMillis());
		code.setExpiredTime(code.getBuildTime() + timeout);
		code.setKey(key);
		code.setValue(val);
		tokens.put(key, code);
	}

	public static boolean match(String key, String val, boolean removed) {
		boolean result = false;
		StateCode<String, String> code = tokens.get(key);
		if (code != null) {
			if (System.currentTimeMillis() < code.getExpiredTime()) {
				result = code.getValue().equals(val);
				if (removed) {
					tokens.remove(key, code);
				}
			}
		}
		return result;
	}

	public void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		response.setHeader("Pragma", "No-cache");
		response.setHeader("Cache-Control", "no-cache");
		response.setDateHeader("Expires", 0);
		response.setContentType("image/jpeg");

		// 生成随机字符串
		String verifyCode = VerifyCodeUtils.generateVerifyCode(4);
		String token = request.getParameter("token");
		if (token != null && token.length() == 32) {
			put(token, verifyCode);
		} else {
			HttpSession session = request.getSession();
			// 将验证码设置到session里 方便验证
			session.setAttribute("verification", verifyCode);
		}
		// 生成图片
		int w = 100, h = 40;
		VerifyCodeUtils.outputImage(w, h, response.getOutputStream(), verifyCode);
	}

	@Override
	public void init() throws ServletException {
		long longValue = 0;
		String tmp = this.getServletConfig().getInitParameter("timeout");
		try {
			longValue = Long.parseLong(tmp);
			if (longValue > (1000 * 90)) {
				PictureVC.timeout = longValue;
			}
		} catch (Throwable th) {
		}

		tmp = this.getServletConfig().getInitParameter("cleanInterval");
		try {
			longValue = Long.parseLong(tmp);
			if (longValue > (1000)) {
				PictureVC.interval = longValue;
			}
		} catch (Throwable th) {
		}
		JfwAppContext.getScheduledExecutorService().scheduleAtFixedRate(new Runnable() {

			@Override
			public void run() {
				LinkedList<StateCode<String, String>> list = new LinkedList<StateCode<String, String>>();
				long time = System.currentTimeMillis();
				for (StateCode<String, String> st : tokens.values()) {
					if ((time - st.getExpiredTime()) > 1000) {
						list.add(st);
					}
				}
				for (StateCode<String, String> st : list) {
					tokens.remove(st.getKey(), st);
				}
			}
		}, 1000, PictureVC.interval, TimeUnit.MILLISECONDS);

	}

}
